#include <string.h>
#include <fstream.h>
#include <stdio.h>
#include <math.h>
#include <X11/IntrinsicP.h>
#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/sensors/SoIdleSensor.h>
#include <Inventor/Xt/SoXtRenderArea.h>
#include <Inventor/actions/SoGLRenderAction.h>
#include <Inventor/nodes/SoPointLight.h>
#include <Inventor/nodes/SoSphere.h>

#include "UserBody.h++"
#include "Person.h++"
#include "makeAliveCamera.h++"


#define ZRANGE 70
#define MAXDISTANCE 45
#define TORSOLENGTH 6
#define PI 3.1416
#define DISPUNIT 1
#define FOOTOFFSET_Y 9
#define FOOTOFFSET_Z 3


SbViewportRegion region(640,480);
// SoGLRenderAction *renderAction(region);

/* units that mesure distance between wall and screen in Alive coords */
/* Z is from the wall to screen */
/* Y is from ceiling to floor */
/* X from edgware to glass */
/* origin is in the middle of Alive space */
// ZRANGE 70 
// YRANGE 60 
// XRANGE 80 

static UserBody* thisbody;
static Person* user;


static 
char *
stringTogether( float feature_x_coord, float feature_y_coord, float feature_z_coord)
{
    char *feature_x=NULL,*feature_y=NULL,*feature_z=NULL,*all_features=NULL;

    feature_x = (char *)calloc(3, sizeof(char));
    feature_y = (char *)calloc(3, sizeof(char));
    feature_z = (char *)calloc(3, sizeof(char));

    all_features = (char *)calloc(11, sizeof(char));

    sprintf(feature_x,"%d",(int) feature_x_coord );
    sprintf(feature_y,"%d",(int) feature_y_coord );
    sprintf(feature_z,"%d",(int) feature_z_coord );

    strcat(all_features,feature_x);
    strcat(all_features,",");
    strcat(all_features,feature_y);
    strcat(all_features,",");
    strcat(all_features,feature_z);

    free(feature_x);
    free(feature_y);
    free(feature_z);

    free(all_features);
    return all_features;
}



static
void
handleEvents()
{
    XtInputMask m;

    while( m = XtAppPending( SoXt::getAppContext() ) )
       XtAppProcessEvent( SoXt::getAppContext(), m );

    user->update();

    if (user->User_exists)
       {
	 
	 thisbody -> setLocation( UserBody::RHAND, 
				  user->right[0],
				  user->right[1],
				  user->right[2]);
	 thisbody -> setLocation( UserBody::LHAND, 
				  user->left[0],
				  user->left[1],
				  user->left[2]);
	 thisbody -> setLocation( UserBody::HEAD, 
				  user->head[0],
				  user->head[1],
				  user->head[2]);
	 thisbody -> setLocation( UserBody::CHEST, 
				  user->chest[0],
				  user->chest[1],
				  user->chest[2]);
	 thisbody -> setLocation( UserBody::WAIST, 
				  user->waist[0],
				  user->waist[1],
				  user->waist[2]);
	 thisbody -> setLocation( UserBody::RFOOT, 
				  user->rfoot[0],
				  user->rfoot[1],
				  user->rfoot[2]);
	 thisbody -> setLocation( UserBody::LFOOT, 
				  user->lfoot[0],
				  user->lfoot[1],
				  user->lfoot[2]);

	 thisbody -> setColor( UserBody::RHAND, 
			       user->right_color[0],
			       user->right_color[1],
			       user->right_color[2]);
	 thisbody -> setColor( UserBody::LHAND, 
			       user->left_color[0],
			       user->left_color[1],
			       user->left_color[2]);
	 thisbody -> setColor( UserBody::HEAD, 
			       user->head_color[0],
			       user->head_color[1],
			       user->head_color[2]);
	 thisbody -> setColor( UserBody::CHEST, 
			       user->chest_color[0],
			       user->chest_color[1],
			       user->chest_color[2]);
	 thisbody -> setColor( UserBody::WAIST, 
			       user->waist_color[0],
			       user->waist_color[1],
			       user->waist_color[2]);
	 thisbody -> setColor( UserBody::RFOOT, 
			       user->rfoot_color[0],
			       user->rfoot_color[1],
			       user->rfoot_color[2]);
	 thisbody -> setColor( UserBody::LFOOT, 
			       user->lfoot_color[0],
			       user->lfoot_color[1],
			       user->lfoot_color[2]);

	 thisbody -> setSize( UserBody::RHAND, 
				  user->right_scale[0],
				  user->right_scale[1]);
	 thisbody -> setSize( UserBody::LHAND, 
				  user->left_scale[0],
				  user->left_scale[1]);
	 thisbody -> setSize( UserBody::HEAD, 
				  user->head_scale[0],
				  user->head_scale[1]);
	 thisbody -> setSize( UserBody::CHEST, 
				  user->chest_scale[0],
				  user->chest_scale[1]);
	 thisbody -> setSize( UserBody::WAIST, 
				  user->waist_scale[0],
				  user->waist_scale[1]);
	 thisbody -> setSize( UserBody::RFOOT, 
				  user->rfoot_scale[0],
				  user->rfoot_scale[1]);
	 thisbody -> setSize( UserBody::LFOOT, 
				  user->lfoot_scale[0],
				  user->lfoot_scale[1]);

	 thisbody -> setOrientation( UserBody::RHAND, 
				     user->right_angle);
	 thisbody -> setOrientation( UserBody::LHAND, 
				     user->left_angle);
	 thisbody -> setOrientation( UserBody::HEAD, 
				     user->head_angle);
	 thisbody -> setOrientation( UserBody::CHEST, 
				     user->chest_angle);
	 thisbody -> setOrientation( UserBody::WAIST, 
				     user->waist_angle);
	 thisbody -> setOrientation( UserBody::RFOOT, 
				     user->rfoot_angle);
	 thisbody -> setOrientation( UserBody::LFOOT, 
				     user->lfoot_angle);

	 static SbBool last_Extend_left = 0;
	 if (user->Extend_left && last_Extend_left)
	   {
	     thisbody -> setText (UserBody::LHAND,
				  "Grab Object");
	   }
	 else
	   {
	     thisbody -> setText (UserBody::LHAND,
				  "");
	   }
	 last_Extend_left = user->Extend_left;

       }
    else
      {	 thisbody -> setSize( UserBody::RHAND, 0., 0.);
	 thisbody -> setSize( UserBody::LHAND, 0., 0.);
	 thisbody -> setSize( UserBody::HEAD, 0., 0.);
	 thisbody -> setSize( UserBody::CHEST, 0., 0.);
	 thisbody -> setSize( UserBody::WAIST, 0., 0.);
	 thisbody -> setSize( UserBody::RFOOT, 0., 0.);
	 thisbody -> setSize( UserBody::LFOOT, 0., 0.);
      }

}


void handleEventsCB(void *, SoSensor *sensor) {
    handleEvents();
    sensor->schedule();
}


void print_visual (XVisualInfo* vinfo)
{
  if (vinfo != (XVisualInfo *) NULL)
    {
      cerr << "Visual:" << endl;
      cerr << "  visualid " << vinfo->visualid <<endl;
      cerr << "  screen " << vinfo->screen <<endl;
      cerr << "  depth " << vinfo->depth <<endl;
      cerr << "  red_mask " << vinfo->red_mask <<endl;
      cerr << "  green_mask " << vinfo->green_mask <<endl;
      cerr << "  blue_mask " << vinfo->blue_mask <<endl;
      cerr << "  colormap_size " << vinfo->colormap_size <<endl;
      cerr << "  bits_per_rgb " << vinfo->bits_per_rgb <<endl;
    }
  else
    {
      cerr << "Visual: NULL" << endl;
    }
}

void
main(int argc, char **argv)
{
  Widget myWindow = SoXt::init(argv[0]);
  if (myWindow == NULL) exit(1); 
 
  //  printf("%s %s\n","name of pfinder host ?", "setenv PFINDER_HOST" );
  user = new Person(getenv("PFINDER_HOST"));
  // if( !thisbody || !user )
  SoPerspectiveCamera* camera = makeAliveCamera( "dogCameraParameters");
  camera -> ref();
  thisbody = new UserBody;

// --------------------------------------------------------------------
// this positions the camera in the center of the Alive screen
// the alive screen is 60 alive coords high so I divide by 2 or subtract 30

//  SbVec3f cameraPos;
//  SoField *cameraPosPoint = camera->getField("position");
//  SoSFVec3f *vecField = (SoSFVec3f *)cameraPosPoint;
//  cameraPos = vecField->getValue();
//  printf("camerapos %f \n",cameraPos[1]); 
//  cameraPos[1] = cameraPos[1] - 30;
//  camera -> position.setValue( cameraPos );
//  printf("camerapos %f \n",cameraPos[1]); 
// --------------------------------------------------------------------

  SoSeparator* root = new SoSeparator;
  root -> ref();

  SoPointLight *mylight = new SoPointLight;
  mylight -> ref();

  SoSphere *sph =  new SoSphere();
  sph->ref();

  root -> addChild( mylight );
  root -> addChild( camera );
  root -> addChild( thisbody->getNode() );


// Set up viewer:
  SoXtExaminerViewer *myViewer = new SoXtExaminerViewer(myWindow);
  myViewer->setSceneGraph( root );
  myViewer->setSize(SbVec2s(725,500));
  myViewer->setPopupMenuEnabled(FALSE);
  myViewer->setDecoration(FALSE);
  myViewer->setHeadlight(TRUE);

  if ((argc > 1) && !strcmp(argv[1],"-chroma"))
    {
      SbColor c;
      float trans;
      c.setPackedValue(0xee00ee00,trans);
      myViewer->setBackgroundColor(c);

      cerr << "chroma transparency set to: " << trans << endl;
    }
  else
    {
      SbColor c;
      float trans;
      c.setPackedValue(0x00000000,trans);
      myViewer->setBackgroundColor(c);

      cerr << "non-chroma transparency set to: " << trans << endl;
    }

//   {
//     int attribList[10] = {GLX_RGBA,
// 			  GLX_TRANSPARENT_TYPE_EXT, 
// 			  GLX_TRANSPARENT_RGB_EXT, 
// 			  None };
//     XVisualInfo* vinfo = glXChooseVisual(myViewer->getDisplay(),
// 					 0,
// 					 attribList );
//     print_visual (vinfo);
//     if (vinfo != (XVisualInfo*) NULL)
//       {
// 	myViewer->setNormalVisual(vinfo);
// 	XFree(vinfo);
//       }
//   }

  myViewer->show();
                 
//   print_visual (myViewer->getNormalVisual());
  SoIdleSensor *idle = new SoIdleSensor(handleEventsCB, NULL);
  idle->schedule();

  SoXt::show(myWindow);
  XtMoveWidget(myWindow , 0, 1);
  SoXt::mainLoop();

}




